@@ -294,7 +294,7 @@ fn wait_for_batches(
 
         debug!("Checking batches via {}", url);
         let request = Client::new()
-            .get(url.clone())
+            .get(url_with_query.clone())
             .header("Authorization", auth.to_string());
         let response = perform_request(request)?;
 
@@ -431,7 +431,7 @@ impl std::fmt::Display for ErrorResponse {
     }
 }
 
-#[cfg(all(test, feature = "rest-api", feature = "rest-api-actix"))]
+#[cfg(all(test, feature = "rest-api-actix-web-1"))]
 mod tests {
     use super::*;
 
