@@ -16,6 +16,7 @@
 //! Hyperledger Transact for state management. Scabbard uses two-phase consensus to reach agreement
 //! on transactions.
 
+#[cfg(feature = "log")]
 #[macro_use]
 extern crate log;
 #[cfg(feature = "diesel")]
@@ -24,16 +25,13 @@ extern crate diesel;
 #[cfg(feature = "diesel_migrations")]
 #[macro_use]
 extern crate diesel_migrations;
-#[macro_use]
-extern crate serde_derive;
 
 #[cfg(feature = "metrics")]
 #[macro_use]
 extern crate metrics;
 
 // pull in `no-op` metric macros if `metrics` is not enabled
-#[cfg(not(feature = "metrics"))]
-#[macro_use]
+#[cfg_attr(all(not(feature = "metrics"), feature = "splinter-service"), macro_use)]
 extern crate splinter;
 
 #[cfg(feature = "client")]
@@ -43,5 +41,6 @@ mod hex;
 pub mod migrations;
 pub mod protocol;
 pub mod protos;
+#[cfg(feature = "splinter-service")]
 pub mod service;
 pub mod store;
