@@ -35,6 +35,7 @@ use sawtooth::receipt::store::ReceiptStore;
 #[cfg(all(feature = "lmdb", any(feature = "postgres", feature = "sqlite")))]
 use splinter::error::InternalError;
 use splinter::error::{InvalidArgumentError, InvalidStateError};
+use splinter::orchestrator::{OrchestratableService, OrchestratableServiceFactory};
 use splinter::service::validation::ServiceArgValidator;
 use splinter::service::{FactoryCreateError, Service, ServiceFactory};
 #[cfg(all(feature = "lmdb", any(feature = "postgres", feature = "sqlite")))]
@@ -43,14 +44,14 @@ use transact::database::Database;
 use transact::state::merkle::sql;
 
 use crate::hex::parse_hex;
-#[cfg(feature = "rest-api-actix")]
+#[cfg(feature = "rest-api-actix-web-1")]
 use crate::service::rest_api::actix;
 #[cfg(all(feature = "lmdb", any(feature = "postgres", feature = "sqlite")))]
 use crate::service::ScabbardStatePurgeHandler;
 #[cfg(any(feature = "postgres", feature = "sqlite"))]
 use crate::service::{
     error::ScabbardError,
-    state::merkle_state::{MerkleState, MerkleStateConfig},
+    state::merkle_state::{self, MerkleState, MerkleStateConfig},
     Scabbard, ScabbardVersion, SERVICE_TYPE,
 };
 #[cfg(feature = "diesel")]
@@ -63,9 +64,6 @@ use crate::store::CommitHashStore;
 #[cfg(all(feature = "lmdb", any(feature = "postgres", feature = "sqlite")))]
 const DEFAULT_LMDB_DIR: &str = "/var/lib/splinter";
 
-#[cfg(any(feature = "postgres", feature = "sqlite"))]
-type ScabbardReceiptStore = Arc<RwLock<dyn ReceiptStore>>;
-
 /// A connection URI to a database instance.
 #[derive(Clone)]
 pub enum ConnectionUri {
@@ -104,6 +102,10 @@ pub enum ScabbardStorageConfiguration {
     Sqlite {
         pool: Pool<ConnectionManager<diesel::SqliteConnection>>,
     },
+    #[cfg(feature = "sqlite")]
+    SqliteExclusiveWrites {
+        pool: Arc<RwLock<Pool<ConnectionManager<diesel::SqliteConnection>>>>,
+    },
 }
 
 impl From<String> for ScabbardStorageConfiguration {
@@ -140,6 +142,15 @@ impl From<Pool<ConnectionManager<diesel::SqliteConnection>>> for ScabbardStorage
     }
 }
 
+#[cfg(feature = "sqlite")]
+impl From<Arc<RwLock<Pool<ConnectionManager<diesel::SqliteConnection>>>>>
+    for ScabbardStorageConfiguration
+{
+    fn from(pool: Arc<RwLock<Pool<ConnectionManager<diesel::SqliteConnection>>>>) -> Self {
+        Self::SqliteExclusiveWrites { pool }
+    }
+}
+
 /// Builds new ScabbardFactory instances.
 #[derive(Default)]
 pub struct ScabbardFactoryBuilder {
@@ -275,11 +286,17 @@ impl ScabbardFactoryBuilder {
             ScabbardStorageConfiguration::Sqlite { pool } => {
                 ScabbardFactoryStorageConfig::Sqlite { pool }
             }
+            #[cfg(feature = "sqlite")]
+            ScabbardStorageConfiguration::SqliteExclusiveWrites { pool } => {
+                ScabbardFactoryStorageConfig::SqliteExclusiveWrites { pool }
+            }
         };
 
         #[cfg(feature = "lmdb")]
         if !state_storage_configuration.enable_lmdb {
             check_for_lmdb_files(lmdb_path)?;
+        } else {
+            check_for_sql_trees(&store_factory_config)?;
         }
 
         #[cfg(feature = "lmdb")]
@@ -365,6 +382,77 @@ fn check_for_lmdb_files(lmdb_path: &Path) -> Result<(), InvalidStateError> {
     }
 }
 
+#[cfg(all(feature = "lmdb", any(feature = "postgres", feature = "sqlite")))]
+fn check_for_sql_trees(
+    store_factory_config: &ScabbardFactoryStorageConfig,
+) -> Result<(), InvalidStateError> {
+    match store_factory_config {
+        #[cfg(feature = "postgres")]
+        ScabbardFactoryStorageConfig::Postgres { pool } => {
+            merkle_state::postgres_list_available_trees(pool)
+                .map_err(|e| {
+                    InvalidStateError::with_message(format!(
+                        "Unable to read merkle state trees in postgres: {}",
+                        e
+                    ))
+                })
+                .and_then(|trees| {
+                    // Check that if any trees exist, it is only the default tree
+                    if trees.iter().any(|name| name != "default") {
+                        Err(InvalidStateError::with_message(
+                            "SQL Merkle Radix trees exist, but LMDB storage is enabled".into(),
+                        ))
+                    } else {
+                        Ok(())
+                    }
+                })
+        }
+        #[cfg(feature = "sqlite")]
+        ScabbardFactoryStorageConfig::Sqlite { pool } => {
+            merkle_state::sqlite_list_available_trees(pool)
+                .map_err(|e| {
+                    InvalidStateError::with_message(format!(
+                        "Unable to read merkle state trees in sqlite: {}",
+                        e
+                    ))
+                })
+                .and_then(|trees| {
+                    // Check that if any trees exist, it is only the default tree
+                    if trees.iter().any(|name| name != "default") {
+                        Err(InvalidStateError::with_message(
+                            "SQL Merkle Radix trees exist, but LMDB storage is enabled".into(),
+                        ))
+                    } else {
+                        Ok(())
+                    }
+                })
+        }
+        #[cfg(feature = "sqlite")]
+        ScabbardFactoryStorageConfig::SqliteExclusiveWrites { pool } => {
+            let pool = pool.read().map_err(|_e| {
+                InvalidStateError::with_message("RwLock on connection pool is poisoned".into())
+            })?;
+            merkle_state::sqlite_list_available_trees(&pool)
+                .map_err(|e| {
+                    InvalidStateError::with_message(format!(
+                        "Unable to read merkle state trees in sqlite: {}",
+                        e
+                    ))
+                })
+                .and_then(|trees| {
+                    // Check that if any trees exist, it is only the default tree
+                    if trees.iter().any(|name| name != "default") {
+                        Err(InvalidStateError::with_message(
+                            "SQL Merkle Radix trees exist, but LMDB storage is enabled".into(),
+                        ))
+                    } else {
+                        Ok(())
+                    }
+                })
+        }
+    }
+}
+
 /// Internal Factory storage configuration.
 #[cfg(any(feature = "postgres", feature = "sqlite"))]
 enum ScabbardFactoryStorageConfig {
@@ -376,6 +464,10 @@ enum ScabbardFactoryStorageConfig {
     Sqlite {
         pool: Pool<ConnectionManager<diesel::SqliteConnection>>,
     },
+    #[cfg(feature = "sqlite")]
+    SqliteExclusiveWrites {
+        pool: Arc<RwLock<Pool<ConnectionManager<diesel::SqliteConnection>>>>,
+    },
 }
 
 pub struct ScabbardFactory {
@@ -476,6 +568,96 @@ impl ServiceFactory for ScabbardFactory {
         circuit_id: &str,
         args: HashMap<String, String>,
     ) -> Result<Box<dyn Service>, FactoryCreateError> {
+        Ok(Box::new(
+            self.create_scabbard(service_id, circuit_id, args)?,
+        ))
+    }
+
+    #[cfg(not(any(feature = "postgres", feature = "sqlite")))]
+    fn create(
+        &self,
+        _service_id: String,
+        _service_type: &str,
+        _circuit_id: &str,
+        _args: HashMap<String, String>,
+    ) -> Result<Box<dyn Service>, FactoryCreateError> {
+        // As the factory cannot be created under these conditions, this function is not reachable.
+        unreachable!()
+    }
+
+    #[cfg(feature = "rest-api")]
+    /// The `Scabbard` services created by the `ScabbardFactory` provide the following REST API
+    /// endpoints as [`ServiceEndpoint`]s:
+    ///
+    /// * `POST /batches` - Add one or more batches to scabbard's queue
+    /// * `GET /batch_statuses` - Get the status of one or more batches
+    /// * `GET /ws/subscribe` - Subscribe to scabbard state-delta events
+    /// * `GET /state/{address}` - Get a value from scabbard's state
+    /// * `GET /state` - Get multiple scabbard state entries
+    /// * `GET /state_root` - Get the current state root hash of scabbard's state
+    ///
+    /// These endpoints are only available if the following REST API backend feature is enabled:
+    ///
+    /// * `rest-api-actix`
+    ///
+    /// [`ServiceEndpoint`]: ../rest_api/struct.ServiceEndpoint.html
+    fn get_rest_endpoints(&self) -> Vec<splinter::service::rest_api::ServiceEndpoint> {
+        // Allowing unused_mut because resources must be mutable if feature rest-api-actix is
+        // enabled
+        #[allow(unused_mut)]
+        let mut endpoints = vec![];
+
+        #[cfg(feature = "rest-api-actix-web-1")]
+        {
+            endpoints.append(&mut vec![
+                actix::batches::make_add_batches_to_queue_endpoint(),
+                actix::ws_subscribe::make_subscribe_endpoint(),
+                actix::batch_statuses::make_get_batch_status_endpoint(),
+                actix::state_address::make_get_state_at_address_endpoint(),
+                actix::state::make_get_state_with_prefix_endpoint(),
+                actix::state_root::make_get_state_root_endpoint(),
+            ])
+        }
+
+        endpoints
+    }
+}
+
+impl OrchestratableServiceFactory for ScabbardFactory {
+    #[cfg(any(feature = "postgres", feature = "sqlite"))]
+    fn create_orchestratable_service(
+        &self,
+        service_id: String,
+        _service_type: &str,
+        circuit_id: &str,
+        args: HashMap<String, String>,
+    ) -> Result<Box<dyn OrchestratableService>, FactoryCreateError> {
+        Ok(Box::new(
+            self.create_scabbard(service_id, circuit_id, args)?,
+        ))
+    }
+
+    #[cfg(not(any(feature = "postgres", feature = "sqlite")))]
+    fn create_orchestratable_service(
+        &self,
+        _service_id: String,
+        _service_type: &str,
+        _circuit_id: &str,
+        _args: HashMap<String, String>,
+    ) -> Result<Box<dyn OrchestratableService>, FactoryCreateError> {
+        // As the factory cannot be created under these conditions, this function is not reachable.
+        unreachable!()
+    }
+}
+
+impl ScabbardFactory {
+    #[cfg(any(feature = "postgres", feature = "sqlite"))]
+    fn create_scabbard(
+        &self,
+        service_id: String,
+        circuit_id: &str,
+        args: HashMap<String, String>,
+    ) -> Result<Scabbard, FactoryCreateError> {
         let peer_services_str = args.get("peer_services").ok_or_else(|| {
             FactoryCreateError::InvalidArguments("peer_services argument not provided".into())
         })?;
@@ -553,14 +735,14 @@ impl ServiceFactory for ScabbardFactory {
             self.create_sql_merkle_state_purge_handle(circuit_id, &service_id),
         );
 
-        let (receipt_store, commit_hash_store): (ScabbardReceiptStore, Box<dyn CommitHashStore>) =
+        let (receipt_store, commit_hash_store): (Arc<dyn ReceiptStore>, Box<dyn CommitHashStore>) =
             match &self.store_factory_config {
                 #[cfg(feature = "postgres")]
                 ScabbardFactoryStorageConfig::Postgres { pool } => (
-                    Arc::new(RwLock::new(DieselReceiptStore::new(
+                    Arc::new(DieselReceiptStore::new(
                         pool.clone(),
                         Some(format!("{}::{}", circuit_id, service_id)),
-                    ))),
+                    )),
                     Box::new(DieselCommitHashStore::new(
                         pool.clone(),
                         circuit_id,
@@ -569,19 +751,31 @@ impl ServiceFactory for ScabbardFactory {
                 ),
                 #[cfg(feature = "sqlite")]
                 ScabbardFactoryStorageConfig::Sqlite { pool } => (
-                    Arc::new(RwLock::new(DieselReceiptStore::new(
+                    Arc::new(DieselReceiptStore::new(
                         pool.clone(),
                         Some(format!("{}::{}", circuit_id, service_id)),
-                    ))),
+                    )),
                     Box::new(DieselCommitHashStore::new(
                         pool.clone(),
                         circuit_id,
                         &service_id,
                     )),
                 ),
+                #[cfg(feature = "sqlite")]
+                ScabbardFactoryStorageConfig::SqliteExclusiveWrites { pool } => (
+                    Arc::new(DieselReceiptStore::new_with_write_exclusivity(
+                        pool.clone(),
+                        Some(format!("{}::{}", circuit_id, service_id)),
+                    )),
+                    Box::new(DieselCommitHashStore::new_with_write_exclusivity(
+                        pool.clone(),
+                        circuit_id,
+                        &service_id,
+                    )),
+                ),
             };
 
-        let service = Scabbard::new(
+        Scabbard::new(
             service_id,
             circuit_id,
             version,
@@ -599,62 +793,9 @@ impl ServiceFactory for ScabbardFactory {
             admin_keys,
             coordinator_timeout,
         )
-        .map_err(|err| FactoryCreateError::CreationFailed(Box::new(err)))?;
-
-        Ok(Box::new(service))
+        .map_err(|err| FactoryCreateError::CreationFailed(Box::new(err)))
     }
 
-    #[cfg(not(any(feature = "postgres", feature = "sqlite")))]
-    fn create(
-        &self,
-        _service_id: String,
-        _service_type: &str,
-        _circuit_id: &str,
-        _args: HashMap<String, String>,
-    ) -> Result<Box<dyn Service>, FactoryCreateError> {
-        // As the factory cannot be created under these conditions, this function is not reachable.
-        unreachable!()
-    }
-
-    #[cfg(feature = "rest-api")]
-    /// The `Scabbard` services created by the `ScabbardFactory` provide the following REST API
-    /// endpoints as [`ServiceEndpoint`]s:
-    ///
-    /// * `POST /batches` - Add one or more batches to scabbard's queue
-    /// * `GET /batch_statuses` - Get the status of one or more batches
-    /// * `GET /ws/subscribe` - Subscribe to scabbard state-delta events
-    /// * `GET /state/{address}` - Get a value from scabbard's state
-    /// * `GET /state` - Get multiple scabbard state entries
-    /// * `GET /state_root` - Get the current state root hash of scabbard's state
-    ///
-    /// These endpoints are only available if the following REST API backend feature is enabled:
-    ///
-    /// * `rest-api-actix`
-    ///
-    /// [`ServiceEndpoint`]: ../rest_api/struct.ServiceEndpoint.html
-    fn get_rest_endpoints(&self) -> Vec<splinter::service::rest_api::ServiceEndpoint> {
-        // Allowing unused_mut because resources must be mutable if feature rest-api-actix is
-        // enabled
-        #[allow(unused_mut)]
-        let mut endpoints = vec![];
-
-        #[cfg(feature = "rest-api-actix")]
-        {
-            endpoints.append(&mut vec![
-                actix::batches::make_add_batches_to_queue_endpoint(),
-                actix::ws_subscribe::make_subscribe_endpoint(),
-                actix::batch_statuses::make_get_batch_status_endpoint(),
-                actix::state_address::make_get_state_at_address_endpoint(),
-                actix::state::make_get_state_with_prefix_endpoint(),
-                actix::state_root::make_get_state_root_endpoint(),
-            ])
-        }
-
-        endpoints
-    }
-}
-
-impl ScabbardFactory {
     /// Check that the LMDB files doesn't exist for the given service.
     #[cfg(feature = "lmdb")]
     #[cfg(all(feature = "lmdb", any(feature = "postgres", feature = "sqlite")))]
@@ -711,6 +852,13 @@ impl ScabbardFactory {
                 pool: pool.clone(),
                 tree_name: format!("{}::{}", circuit_id, service_id),
             },
+            #[cfg(feature = "sqlite")]
+            ScabbardFactoryStorageConfig::SqliteExclusiveWrites { pool } => {
+                MerkleStateConfig::SqliteExclusiveWrites {
+                    pool: pool.clone(),
+                    tree_name: format!("{}::{}", circuit_id, service_id),
+                }
+            }
         }
     }
 
@@ -730,6 +878,13 @@ impl ScabbardFactory {
             }
             #[cfg(feature = "sqlite")]
             ScabbardFactoryStorageConfig::Sqlite { pool } => {
+                Box::new(SqliteMerkleStatePurgeHandler {
+                    pool: Arc::new(RwLock::new(pool.clone())),
+                    tree_name: format!("{}::{}", circuit_id, service_id),
+                })
+            }
+            #[cfg(feature = "sqlite")]
+            ScabbardFactoryStorageConfig::SqliteExclusiveWrites { pool } => {
                 Box::new(SqliteMerkleStatePurgeHandler {
                     pool: pool.clone(),
                     tree_name: format!("{}::{}", circuit_id, service_id),
@@ -822,7 +977,7 @@ impl ScabbardStatePurgeHandler for PostgresMerkleStatePurgeHandler {
 
 #[cfg(feature = "sqlite")]
 struct SqliteMerkleStatePurgeHandler {
-    pool: Pool<ConnectionManager<diesel::SqliteConnection>>,
+    pool: Arc<RwLock<Pool<ConnectionManager<diesel::SqliteConnection>>>>,
     tree_name: String,
 }
 
