@@ -27,7 +27,7 @@ mod state;
 use std::any::Any;
 use std::collections::{HashSet, VecDeque};
 use std::convert::TryFrom;
-use std::sync::{Arc, Mutex, RwLock};
+use std::sync::{Arc, Mutex};
 use std::time::Duration;
 
 use cylinder::Verifier as SignatureVerifier;
@@ -35,6 +35,7 @@ use protobuf::Message;
 use sawtooth::receipt::store::ReceiptStore;
 use splinter::{
     consensus::{Proposal, ProposalUpdate},
+    orchestrator::OrchestratableService,
     service::{
         Service, ServiceDestroyError, ServiceError, ServiceMessageContext, ServiceNetworkRegistry,
         ServiceStartError, ServiceStopError,
@@ -85,8 +86,6 @@ impl TryFrom<Option<&str>> for ScabbardVersion {
     }
 }
 
-type ScabbardReceiptStore = Arc<RwLock<dyn ReceiptStore>>;
-
 /// A handler for purging a scabbard instances state
 pub trait ScabbardStatePurgeHandler: Send + Sync {
     /// Purge the scabbard instances state.
@@ -119,7 +118,7 @@ impl Scabbard {
         peer_services: HashSet<String>,
         merkle_state: MerkleState,
         commit_hash_store: Box<dyn CommitHashStore>,
-        receipt_store: ScabbardReceiptStore,
+        receipt_store: Arc<dyn ReceiptStore>,
         purge_handler: Box<dyn ScabbardStatePurgeHandler>,
         signature_verifier: Box<dyn SignatureVerifier>,
         // The public keys that are authorized to create and manage sabre contracts
@@ -512,6 +511,16 @@ impl Service for Scabbard {
     }
 }
 
+impl OrchestratableService for Scabbard {
+    fn as_service(&self) -> &dyn Service {
+        self
+    }
+
+    fn clone_box(&self) -> Box<dyn OrchestratableService> {
+        Box::new(self.clone())
+    }
+}
+
 #[cfg(test)]
 pub mod tests {
     use super::*;
@@ -545,7 +554,7 @@ pub mod tests {
             HashSet::new(),
             merkle_state,
             commit_hash_store,
-            Arc::new(RwLock::new(MockReceiptStore)),
+            Arc::new(MockReceiptStore),
             Box::new(NoOpScabbardStatePurgeHandler),
             Secp256k1Context::new().new_verifier(),
             vec![],
@@ -569,7 +578,7 @@ pub mod tests {
             HashSet::new(),
             merkle_state,
             commit_hash_store,
-            Arc::new(RwLock::new(MockReceiptStore)),
+            Arc::new(MockReceiptStore),
             Box::new(NoOpScabbardStatePurgeHandler),
             Secp256k1Context::new().new_verifier(),
             vec![],
@@ -592,7 +601,7 @@ pub mod tests {
             HashSet::new(),
             merkle_state,
             commit_hash_store,
-            Arc::new(RwLock::new(MockReceiptStore)),
+            Arc::new(MockReceiptStore),
             Box::new(NoOpScabbardStatePurgeHandler),
             Secp256k1Context::new().new_verifier(),
             vec![],
