@@ -100,7 +100,7 @@ pub fn make_get_state_with_prefix_endpoint() -> ServiceEndpoint {
 mod tests {
     use super::*;
 
-    use std::sync::{Arc, Mutex, RwLock};
+    use std::sync::{Arc, Mutex};
 
     use cylinder::{secp256k1::Secp256k1Context, Context};
     use diesel::r2d2::{ConnectionManager, Pool};
@@ -113,11 +113,8 @@ mod tests {
         state::merkle::INDEXES,
     };
     use transact::{
-        families::command::make_command_transaction,
-        protocol::{
-            batch::BatchBuilder,
-            command::{BytesEntry, Command, SetState},
-        },
+        families::command::workload::CommandTransactionBuilder,
+        protocol::command::{BytesEntry, Command, SetState},
     };
 
     #[cfg(feature = "authorization")]
@@ -166,10 +163,10 @@ mod tests {
     fn state_with_prefix() {
         let (merkle_state, commit_hash_store) = create_merkle_state_and_commit_hash_store();
 
-        let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(
+        let receipt_store = Arc::new(DieselReceiptStore::new(
             create_connection_pool_and_migrate(":memory:".to_string()),
             None,
-        )));
+        ));
 
         // Initialize a temporary scabbard state and set some values; this will pre-populate the DBs
         let prefix = "abcdef".to_string();
@@ -194,19 +191,16 @@ mod tests {
 
             let signing_context = Secp256k1Context::new();
             let signer = signing_context.new_signer(signing_context.new_random_private_key());
-            let batch = BatchBuilder::new()
-                .with_transactions(vec![
-                    make_command_transaction(
-                        &[Command::SetState(SetState::new(vec![
-                            BytesEntry::new(address1.clone(), value1.clone()),
-                            BytesEntry::new(address2.clone(), value2.clone()),
-                            BytesEntry::new(address3.clone(), value3.clone()),
-                        ]))],
-                        &*signer,
-                    )
-                    .take()
-                    .0,
-                ])
+            let batch = CommandTransactionBuilder::new()
+                .with_commands(vec![Command::SetState(SetState::new(vec![
+                    BytesEntry::new(address1.clone(), value1.clone()),
+                    BytesEntry::new(address2.clone(), value2.clone()),
+                    BytesEntry::new(address3.clone(), value3.clone()),
+                ]))])
+                .into_transaction_builder()
+                .expect("failed to convert to transaction builder")
+                .into_batch_builder(&*signer)
+                .expect("failed to build transaction")
                 .build_pair(&*signer)
                 .expect("Failed to build batch");
             state
