@@ -18,15 +18,16 @@ use std::collections::{HashMap, HashSet, VecDeque};
 use std::convert::TryFrom;
 use std::fmt;
 use std::sync::{
-    mpsc::{channel, Receiver, RecvTimeoutError, Sender},
-    Arc, RwLock,
+    mpsc::{channel, Receiver, Sender, TryRecvError},
+    Arc,
 };
 use std::time::{Duration, Instant, SystemTime};
 
 use protobuf::Message;
 use sawtooth::receipt::store::ReceiptStore;
 use sawtooth_sabre::{
-    handler::SabreTransactionHandler, ADMINISTRATORS_SETTING_ADDRESS, ADMINISTRATORS_SETTING_KEY,
+    admin::SettingsAdminPermission, handler::SabreTransactionHandler,
+    ADMINISTRATORS_SETTING_ADDRESS, ADMINISTRATORS_SETTING_KEY,
 };
 use serde::{Deserialize, Serialize};
 #[cfg(feature = "events")]
@@ -54,11 +55,9 @@ use crate::store::CommitHashStore;
 
 const EXECUTION_TIMEOUT: u64 = 300; // five minutes
 const ITER_CACHE_SIZE: usize = 64;
-const COMPLETED_BATCH_INFO_ITER_RETRY_MILLIS: u64 = 100;
+const COMPLETED_BATCH_INFO_ITER_RETRY: Duration = Duration::from_millis(100);
 const DEFAULT_BATCH_HISTORY_SIZE: usize = 100;
 
-type ScabbardReceiptStore = Arc<RwLock<dyn ReceiptStore>>;
-
 /// Iterator over entries in a Scabbard service's state
 pub type StateIter = Box<dyn Iterator<Item = Result<(String, Vec<u8>), ScabbardStateError>>>;
 
@@ -68,7 +67,7 @@ pub struct ScabbardState {
     context_manager: ContextManager,
     executor: Executor,
     current_state_root: String,
-    receipt_store: ScabbardReceiptStore,
+    receipt_store: Arc<dyn ReceiptStore>,
     pending_changes: Option<(String, Vec<TransactionReceipt>)>,
     event_subscribers: Vec<Box<dyn StateSubscriber>>,
     #[cfg(feature = "metrics")]
@@ -82,7 +81,7 @@ impl ScabbardState {
     pub fn new(
         merkle_state: merkle_state::MerkleState,
         commit_hash_store: Box<dyn CommitHashStore>,
-        receipt_store: ScabbardReceiptStore,
+        receipt_store: Arc<dyn ReceiptStore>,
         #[cfg(feature = "metrics")] service_id: String,
         #[cfg(feature = "metrics")] circuit_id: String,
         admin_keys: Vec<String>,
@@ -125,7 +124,7 @@ impl ScabbardState {
         let mut executor = Executor::new(vec![Box::new(StaticExecutionAdapter::new_adapter(
             vec![
                 Box::new(SawtoothToTransactHandlerAdapter::new(
-                    SabreTransactionHandler::new(),
+                    SabreTransactionHandler::new(Box::new(SettingsAdminPermission)),
                 )),
                 #[cfg(test)]
                 Box::new(CommandTransactionHandler::new()),
@@ -288,13 +287,6 @@ impl ScabbardState {
                     .collect::<Result<Vec<_>, _>>()?;
 
                 self.receipt_store
-                    .write()
-                    .map_err(|err| {
-                        ScabbardStateError(format!(
-                            "transaction receipt store lock poisoned: {}",
-                            err
-                        ))
-                    })?
                     .add_txn_receipts(txn_receipts)
                     .map_err(|err| {
                         ScabbardStateError(format!(
@@ -455,14 +447,14 @@ enum EventQuery {
 /// An iterator that wraps the `ReceiptStore` and returns `StateChangeEvent`s using an
 /// in-memory cache.
 pub struct Events {
-    receipt_store: Arc<RwLock<dyn ReceiptStore>>,
+    receipt_store: Arc<dyn ReceiptStore>,
     query: EventQuery,
     cache: VecDeque<StateChangeEvent>,
 }
 
 impl Events {
     fn new(
-        receipt_store: Arc<RwLock<dyn ReceiptStore>>,
+        receipt_store: Arc<dyn ReceiptStore>,
         start_id: Option<String>,
     ) -> Result<Self, ScabbardStateError> {
         let mut iter = Events {
@@ -477,14 +469,10 @@ impl Events {
     fn reload_cache(&mut self) -> Result<(), ScabbardStateError> {
         match self.query {
             EventQuery::Fetch(ref start_id) => {
-                let receipt_store = self.receipt_store.read().map_err(|err| {
-                    ScabbardStateError(format!("transaction receipt store lock poisoned: {}", err))
-                })?;
-
                 self.cache = if let Some(id) = start_id.as_ref() {
-                    receipt_store.list_receipts_since(Some(id.clone()))
+                    self.receipt_store.list_receipts_since(Some(id.clone()))
                 } else {
-                    receipt_store.list_receipts_since(None)
+                    self.receipt_store.list_receipts_since(None)
                 }
                 .map_err(|err| {
                     ScabbardStateError(format!(
@@ -633,7 +621,7 @@ impl BatchHistory {
         let batch_info = self.upsert_batch(signature.into(), status);
 
         match batch_info.status {
-            BatchStatus::Invalid(_) | BatchStatus::Valid(_) => {
+            BatchStatus::Invalid(_) | BatchStatus::Committed(_) => {
                 self.send_completed_batch_info_to_subscribers(batch_info)
             }
             _ => {}
@@ -644,7 +632,7 @@ impl BatchHistory {
         match self.history.get_mut(signature) {
             Some(info) => match info.status.clone() {
                 BatchStatus::Valid(txns) => {
-                    info.set_status(BatchStatus::Committed(txns));
+                    self.update_batch_status(signature, BatchStatus::Committed(txns));
                 }
                 _ => {
                     error!(
@@ -755,9 +743,15 @@ impl BatchHistory {
             .batch_subscribers
             .drain(..)
             .filter_map(|(mut pending_signatures, sender)| {
-                if pending_signatures.remove(&info.id) && sender.send(info.clone()).is_err() {
-                    // Receiver has been dropped
-                    return None;
+                match info.status {
+                    BatchStatus::Invalid(_) | BatchStatus::Committed(_) => {
+                        if pending_signatures.remove(&info.id) && sender.send(info.clone()).is_err()
+                        {
+                            // Receiver has been dropped
+                            return None;
+                        }
+                    }
+                    _ => (),
                 }
 
                 if pending_signatures.is_empty() {
@@ -795,14 +789,12 @@ impl ChannelBatchInfoIter {
         timeout: Duration,
         pending_ids: HashSet<String>,
     ) -> Result<Self, ScabbardStateError> {
-        let timeout = Instant::now()
-            .checked_add(timeout)
-            .ok_or_else(|| ScabbardStateError("failed to schedule timeout".into()))?;
-
         Ok(Self {
             receiver,
-            retry_interval: Duration::from_millis(COMPLETED_BATCH_INFO_ITER_RETRY_MILLIS),
-            timeout,
+            retry_interval: std::cmp::min(timeout, COMPLETED_BATCH_INFO_ITER_RETRY),
+            timeout: Instant::now()
+                .checked_add(timeout)
+                .ok_or_else(|| ScabbardStateError("failed to schedule timeout".into()))?,
             pending_ids,
         })
     }
@@ -817,21 +809,26 @@ impl Iterator for ChannelBatchInfoIter {
             if self.pending_ids.is_empty() {
                 return None;
             }
-            // Check if the timeout has expired
-            if Instant::now() >= self.timeout {
-                return Some(Err(format!(
-                    "timeout expired while waiting for incompleted batches: {:?}",
-                    self.pending_ids
-                )));
-            }
-            // Check for the next BatchInfo
-            match self.receiver.recv_timeout(self.retry_interval) {
-                Ok(batch_info) => {
-                    self.pending_ids.remove(&batch_info.id);
-                    return Some(Ok(batch_info));
+
+            match self.receiver.try_recv() {
+                Ok(batch_info) => match batch_info.status {
+                    BatchStatus::Invalid(_) | BatchStatus::Committed(_) => {
+                        self.pending_ids.remove(&batch_info.id);
+                        return Some(Ok(batch_info));
+                    }
+                    _ => {}
+                },
+                Err(TryRecvError::Empty) => {
+                    // Check if the timeout has expired
+                    if Instant::now() >= self.timeout {
+                        return Some(Err(format!(
+                            "timeout expired while waiting for incompleted batches: {:?}",
+                            self.pending_ids
+                        )));
+                    }
+                    std::thread::sleep(self.retry_interval);
                 }
-                Err(RecvTimeoutError::Timeout) => {}
-                Err(RecvTimeoutError::Disconnected) => return None,
+                Err(TryRecvError::Disconnected) => return None,
             }
         }
     }
@@ -851,11 +848,8 @@ mod tests {
     use sawtooth::receipt::store::diesel::DieselReceiptStore;
     use transact::{
         database::{btree::BTreeDatabase, Database},
-        families::command::make_command_transaction,
-        protocol::{
-            batch::BatchBuilder,
-            command::{BytesEntry, Command, SetState},
-        },
+        families::command::workload::CommandTransactionBuilder,
+        protocol::command::{BytesEntry, Command, SetState},
         state::merkle::INDEXES,
     };
 
@@ -863,15 +857,111 @@ mod tests {
 
     use super::merkle_state::{MerkleState, MerkleStateConfig};
 
+    /// Verify that the ChannelBatchInfoIter returns an error if no results are returned before the
+    /// timeout.
+    #[test]
+    fn channel_batch_iter_no_resuls_before_timeout() -> Result<(), Box<dyn std::error::Error>> {
+        let (_tx, rx) = channel();
+
+        let mut iter = ChannelBatchInfoIter::new(
+            rx,
+            Duration::from_secs(0),
+            vec!["batch-id-1".to_string()].into_iter().collect(),
+        )?;
+
+        assert!(iter.next().unwrap().is_err());
+
+        Ok(())
+    }
+
+    /// Verify that the ChannelBatchInfoIter returns a value if it is sent before next is called.
+    #[test]
+    fn channel_batch_iter_batch_info_before_next() -> Result<(), Box<dyn std::error::Error>> {
+        let (tx, rx) = channel();
+
+        let mut iter = ChannelBatchInfoIter::new(
+            rx,
+            Duration::from_secs(0),
+            vec!["batch-id-1".to_string()].into_iter().collect(),
+        )?;
+
+        tx.send(BatchInfo {
+            id: "batch-id-1".into(),
+            status: BatchStatus::Committed(vec![ValidTransaction::new("ab".into())]),
+            timestamp: SystemTime::now(),
+        })?;
+
+        let info = iter.next().transpose()?;
+
+        assert!(info.is_some());
+
+        let info = info.unwrap();
+
+        assert_eq!(&info.id, "batch-id-1");
+
+        let info = iter.next();
+
+        assert!(info.is_none());
+
+        Ok(())
+    }
+
+    /// Verify that the ChannelBatchInfoIter returns a value if it is sent after next is called.
+    #[test]
+    fn channel_batch_iter_batch_info_after_next() -> Result<(), Box<dyn std::error::Error>> {
+        let (tx, rx) = channel();
+
+        let barrier = Arc::new(std::sync::Barrier::new(2));
+
+        let iter_barrier = Arc::clone(&barrier);
+        let jh = std::thread::spawn(move || {
+            let mut iter = ChannelBatchInfoIter::new(
+                rx,
+                Duration::from_secs(1),
+                vec!["batch-id-1".to_string()].into_iter().collect(),
+            )
+            .unwrap();
+
+            iter_barrier.wait();
+
+            let info = iter.next().transpose().unwrap();
+
+            assert!(info.is_some());
+
+            let info = info.unwrap();
+
+            assert_eq!(&info.id, "batch-id-1");
+
+            let info = iter.next();
+
+            assert!(info.is_none());
+        });
+
+        barrier.wait();
+
+        // Wait a tiny amount for the iter.next() call.
+        std::thread::sleep(Duration::from_millis(10));
+
+        tx.send(BatchInfo {
+            id: "batch-id-1".into(),
+            status: BatchStatus::Committed(vec![ValidTransaction::new("ab".into())]),
+            timestamp: SystemTime::now(),
+        })?;
+
+        jh.join().unwrap();
+
+        Ok(())
+    }
+
     /// Verify that an empty receipt store returns an empty iterator
     #[test]
     fn empty_event_iterator() {
         let pool = create_connection_pool_and_migrate(":memory:".to_string());
 
-        let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(
+        let receipt_store = Arc::new(DieselReceiptStore::new(
             pool,
             Some("empty_event_iterator".into()),
-        )));
+        ));
 
         // Test without a specified start
         let all_events =
@@ -899,14 +989,9 @@ mod tests {
 
         let pool = create_connection_pool_and_migrate(":memory:".to_string());
 
-        let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(
-            pool,
-            Some("event_iterator".into()),
-        )));
+        let receipt_store = Arc::new(DieselReceiptStore::new(pool, Some("event_iterator".into())));
 
         receipt_store
-            .write()
-            .expect("failed to get write lock")
             .add_txn_receipts(receipts.clone())
             .expect("failed to add receipts to store");
 
@@ -937,10 +1022,10 @@ mod tests {
     #[test]
     fn get_state_at_address() {
         // Initialize state
-        let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(
+        let receipt_store = Arc::new(DieselReceiptStore::new(
             create_connection_pool_and_migrate(":memory:".to_string()),
             None,
-        )));
+        ));
 
         let db = create_btree_db();
         let merkle_state = MerkleState::new(MerkleStateConfig::key_value(db.clone_box()))
@@ -965,18 +1050,14 @@ mod tests {
 
         let signing_context = Secp256k1Context::new();
         let signer = signing_context.new_signer(signing_context.new_random_private_key());
-        let batch = BatchBuilder::new()
-            .with_transactions(vec![
-                make_command_transaction(
-                    &[Command::SetState(SetState::new(vec![BytesEntry::new(
-                        address.clone(),
-                        value.clone(),
-                    )]))],
-                    &*signer,
-                )
-                .take()
-                .0,
-            ])
+        let batch = CommandTransactionBuilder::new()
+            .with_commands(vec![Command::SetState(SetState::new(vec![
+                BytesEntry::new(address.clone(), value.clone()),
+            ]))])
+            .into_transaction_builder()
+            .expect("failed to convert to transaction builder")
+            .into_batch_builder(&*signer)
+            .expect("failed to build transaction")
             .build_pair(&*signer)
             .expect("Failed to build batch");
         state
@@ -1014,10 +1095,10 @@ mod tests {
     ///    that no entries are returned (the iterator is empty).
     #[test]
     fn get_state_with_prefix() {
-        let receipt_store = Arc::new(RwLock::new(DieselReceiptStore::new(
+        let receipt_store = Arc::new(DieselReceiptStore::new(
             create_connection_pool_and_migrate(":memory:".to_string()),
             None,
-        )));
+        ));
 
         let db = create_btree_db();
         let merkle_state = MerkleState::new(MerkleStateConfig::key_value(db.clone_box()))
@@ -1048,19 +1129,16 @@ mod tests {
 
         let signing_context = Secp256k1Context::new();
         let signer = signing_context.new_signer(signing_context.new_random_private_key());
-        let batch = BatchBuilder::new()
-            .with_transactions(vec![
-                make_command_transaction(
-                    &[Command::SetState(SetState::new(vec![
-                        BytesEntry::new(address1.clone(), value1.clone()),
-                        BytesEntry::new(address2.clone(), value2.clone()),
-                        BytesEntry::new(address3.clone(), value3.clone()),
-                    ]))],
-                    &*signer,
-                )
-                .take()
-                .0,
-            ])
+        let batch = CommandTransactionBuilder::new()
+            .with_commands(vec![Command::SetState(SetState::new(vec![
+                BytesEntry::new(address1.clone(), value1.clone()),
+                BytesEntry::new(address2.clone(), value2.clone()),
+                BytesEntry::new(address3.clone(), value3.clone()),
+            ]))])
+            .into_transaction_builder()
+            .expect("failed to convert to transaction builder")
+            .into_batch_builder(&*signer)
+            .expect("failed to build transaction")
             .build_pair(&*signer)
             .expect("Failed to build batch");
         state
